%{

#include <stdio.h>

%}

%start test

/* You won't see these tokens
 * %token EOF NIL ECOMM TCOMM
 * %token ASSIGN COLON COMMA DIGITS DIV DOT LANG LBRACK LPAREN
 * %token MINUS MUL NAME PERC PLUS POW RANG RBRACK RPAREN SEMI STRING
 */

/* These tokens will be recognized by the lexical analyzer */
%token T_DIGITS T_NAME T_STRING

/* These tokens are reserved keywords */
%token K_ADDRESSING_MODE K_ALIGN K_ALWAYS K_AND K_BEGIN K_BIND K_BIT
%token K_BUILTIN K_BY K_BYTE K_CASE K_CODECOMMENT K_COMPILETIME K_DECR
%token K_DECRA K_DECRU K_DO K_ELSE K_ELUDOM K_ENABLE K_END K_EQL
%token K_EQLA K_EQLU K_EQV K_EXITLOOP K_EXTERNAL K_FIELD K_FORWARD
%token K_FROM K_GEQ K_GEQA K_GEQU K_GLOBAL K_GTR K_GTRA K_GTRU K_IF
%token K_INCR K_INCRA K_INCRU K_INITIAL K_INRANGE K_IOPAGE
%token K_KEYWORDMACRO K_LABEL K_LEAVE K_LEQ K_LEQA K_LEQU K_LIBRARY
%token K_LINKAGE K_LITERAL K_LOCAL K_LONG K_LSS K_LSSA K_LSSU K_MACRO
%token K_MAP K_MOD K_MODULE K_NEQ K_NEQA K_NEQU K_NOT K_NOVALUE K_OF
%token K_OR K_OTHERWISE K_OUTRANGE K_OWN K_PLIT K_PRESET K_PSECT
%token K_RECORD K_REF K_REGISTER K_REP K_REQUIRE K_RETURN K_ROUTINE
%token K_SELECT K_SELECTA K_SELECTONE K_SELECTONEA K_SELECTONEU
%token K_SELECTU K_SET K_SHOW K_SIGNED K_STACKLOCAL K_STRUCTURE
%token K_SWITCHES K_TES K_THEN K_TO K_UNDECLARE K_UNSIGNED K_UNTIL
%token K_UPLIT K_VOLATILE K_WEAK K_WHILE K_WITH K_WORD K_XOR

/* These tokens are reserved keywords with a leading percent */  
%token P_ALLOCATION P_ASCIC P_ASCID P_ASCII P_ASCIZ P_ASSIGN P_B
%token P_BLISS P_BLISS16 P_BLISS32 P_BLISS36 P_BPADDR P_BPUNIT P_BPVAL
%token P_C P_CHAR P_CHARCOUNT P_COUNT P_CTCE P_D P_DECIMAL P_DECLARED
%token P_CHESC
%token P_E P_ELSE P_ERROR P_ERRORMACRO P_EXACTSTRING P_EXITITERATION
%token P_EXITMACRO P_EXPAND P_EXPLODE P_FI P_FIELDEXPAND P_G P_H
%token P_IDENTICAL P_IF P_INFORM P_ISSTRING P_LENGTH P_LTCE P_MESSAGE
%token P_NAME P_NBITS P_NBITSU P_NULL P_NUMBER P_O P_P P_PRINT P_QUOTE
%token P_QUOTENAME P_RAD50_10 P_RAD50_11 P_REF P_REMAINING P_REMOVE
%token P_REQUIRE P_SBTTL P_SIXBIT P_SIZE P_STRING P_SWITCHES P_THE
%token P_TITLE P_UNQUOTE P_UPVAL P_VARIANT P_WARN P_X

%%

/**** 1.0 MODULES ***************************************************/

module		: K_MODULE module_head '=' module_body K_ELUDOM
			{
			  fprintf (stderr, "That's a module alright\n");
			}
		;

module_head	: T_NAME
			{ /* set the default switches */
			}
		opt_mhargs
		;

opt_mhargs	: /* empty */
		| '(' ms_list ')'
  		;

ms_list		: module_switch
		| ms_list ',' module_switch
		;

module_body	: /* empty */
		| K_BEGIN decl_list K_END
		| '(' decl_list ')'
		;

decl_list	: declaration
		| decl_list declaration
		;

/* 1.1 Module Switches */

module_switch	: on_off_switch
		| special_switch
		;

on_off_switch	: T_NAME
			{ CODE		/* default */
		| NOCODE
		| DEBUG
		| NODEBUG		/* default */
		| ERRS			/* default */
		| NOERRS
		| OPTIMIZE		/* default */
		| NOOPTIMIZE
		| UNAMES
		| NOUNAMES		/* default */
		| SAFE			/* default */
		| NOSAFE
		| ZIP
		| NOZIP			/* default */
			}
		;

special_switch	: T_NAME '=' T_NAME
			{
			  switch ($1 unres keyw) {
			  case U_MAIN:
			    switch_main = $3;
			    break;
			  default:
			    yyerror ();
			  }
			}
		| T_NAME '=' T_DIGITS
			{
			  switch ($1 unres keyw) {
			  case U_OPTLEVEL:
			    switch_optlevel = $3;
			    /* 0 | 1 | 2 | 3 */
			    break;
			  default:
			    yyerror ();
			  }
			}
		| T_NAME '=' T_STRING
			{
			  switch ($1 unres keyw) {
			  case U_IDENT:
			    switch_ident = $3;
			    break;
			  case U_VERSION:
			    switch_version = $3;
			    break;
			  default:
			    yyerror ();
			  }
			}
		| T_NAME '(' name_list ')'
			{ /* declare p */
			  int i;

			  switch ($1 unres keyw) {
			  case U_LANGUAGE:
			    switch_language = $3;
			    /* language_list */
			    /* U_COMMON, U_BLISSxy */
			    break;
			  case U_LINKAGE:
			    /* linkage_name */
			    break;
			  case U_LIST:
			    /* list_option ,... */
			    i = 1 | 32 | 128 | 256 | 512;
			    for (p=$3; p; p = p->next)
			      {
				switch (*p)
				  {
				  case U_SOURCE:   i |=   1; break;
				  case U_NOSOURCE: i &=  ~1; break;
				  case U_REQUIRE:  i |=   2; break;
				  case U_NOREQUIRE:i &=  ~2; break;
				  case U_EXPAND:   i |=   4; break;
				  case U_NOEXPAND: i &=  ~4; break;
				  case U_TRACE:	   i |=   8; break;
				  case U_NOTRACE:  i &=  ~8; break;
				  case U_LIBRARY:  i |=  16; break;
				  case U_NOLIBRARY:i &= ~16; break;
				  case U_OBJECT:   i |=  32; break;
				  case U_NOOBJECT: i &= ~32; break;
				  case U_ASSEMBLY: i |=  64; break;
				  case U_NOASSEMBLY:i &= ~64; break;
				  case U_SYMBOLIC:  i |= 128; break;
				  case U_NOSYMBOLIC:i &= ~128; break;
				  case U_BINARY:    i |=  256; break;
				  case U_NOBINARY:  i &= ~256; break;
				  case U_COMMENTARY:i |=  512; break;
				  case U_NOCOMMENTARY:
				    i &= ~512; break;
				  default:
				    yyerror ();
				  }
				switch_list = i;
			      }
			  }
			  case U_STRUCTURE:
			    /* optional structure_attribute */
			    break;
			  default:
			    yyerror ();
			}
		;

name_list	: T_NAME
			{
			  $$ = ($1 unres keyw);
			}
		| name_list ',' T_NAME
			{
			  $$ = link($1, ($3 unres keyw));
			}
		| /* empty */
			{
			  $$ = NULL;
			}
		;

/**** 2.0 EXPRESSIONS ***********************************************/
/**** 3.0 CONSTANT EXPRESSIONS **************************************/
/**** 4.0 DECLARATIONS **********************************************/
declaration	: /* empty */
/**** 5.0 LEXICAL PROCESSING FACILITIES *****************************/
/**** 7.0 MACHINE SPECIFIC NAMES ************************************/

test		: tok
		| test tok
		;

tok		: '='
			{ fprintf (stderr, " '=' \n"); }
		| ':'
			{ fprintf (stderr, " ':' \n"); }
		| ','
			{ fprintf (stderr, " ',' \n"); }
		| '/'
			{ fprintf (stderr, " '/' \n"); }
		| '.'
			{ fprintf (stderr, " '.' \n"); }
		| '<'
			{ fprintf (stderr, " '<' \n"); }
		| '['
			{ fprintf (stderr, " '[' \n"); }
		| '('
			{ fprintf (stderr, " '(' \n"); }
		| '-'
			{ fprintf (stderr, " '-' \n"); }
		| '*'
			{ fprintf (stderr, " '*' \n"); }
		| '+'
			{ fprintf (stderr, " '+' \n"); }
		| '^'
			{ fprintf (stderr, " '^' \n"); }
		| '>'
			{ fprintf (stderr, " '>' \n"); }
		| ']'
			{ fprintf (stderr, " ']' \n"); }
		| ')'
			{ fprintf (stderr, " ')' \n"); }
		| ';'
			{ fprintf (stderr, " ';' \n"); }
		| '%'
			{ fprintf (stderr, " '%' \n"); }
		| T_DIGITS
			{ fprintf (stderr, "Integer constant \n"); }
		| T_NAME
			{ fprintf (stderr, "Name \n"); }
		| T_STRING
			{ fprintf (stderr, "Quoted string \n"); }
		| K_ADDRESSING_MODE
			{ fprintf (stderr, "ADDRESSING_MODE \n"); }
		| K_ALIGN
			{ fprintf (stderr, "ALIGN \n"); }
		| K_ALWAYS
			{ fprintf (stderr, "ALWAYS \n"); }
		| K_AND
			{ fprintf (stderr, "AND \n"); }
		| K_BEGIN
			{ fprintf (stderr, "BEGIN \n"); }
		| K_BIND
			{ fprintf (stderr, "BIND \n"); }
		| K_BIT
			{ fprintf (stderr, "BIT \n"); }
		| K_BUILTIN
			{ fprintf (stderr, "BUILTIN \n"); }
		| K_BY
			{ fprintf (stderr, "BY \n"); }
		| K_BYTE
			{ fprintf (stderr, "BYTE \n"); }
		| K_CASE
			{ fprintf (stderr, "CASE \n"); }
		| K_CODECOMMENT
			{ fprintf (stderr, "CODECOMMENT \n"); }
		| K_COMPILETIME
			{ fprintf (stderr, "COMPILETIME \n"); }
		| K_DECR
			{ fprintf (stderr, "DECR \n"); }
		| K_DECRA
			{ fprintf (stderr, "DECRA \n"); }
		| K_DECRU
			{ fprintf (stderr, "DECRU \n"); }
		| K_DO
			{ fprintf (stderr, "DO \n"); }
		| K_ELSE
			{ fprintf (stderr, "ELSE \n"); }
		| K_ELUDOM
			{ fprintf (stderr, "ELUDOM \n"); }
		| K_ENABLE
			{ fprintf (stderr, "ENABLE \n"); }
		| K_END
			{ fprintf (stderr, "END \n"); }
		| K_EQL
			{ fprintf (stderr, "EQL \n"); }
		| K_EQLA
			{ fprintf (stderr, "EQLA \n"); }
		| K_EQLU
			{ fprintf (stderr, "EQLU \n"); }
		| K_EQV
			{ fprintf (stderr, "EQV \n"); }
		| K_EXITLOOP
			{ fprintf (stderr, "EXITLOOP \n"); }
		| K_EXTERNAL
			{ fprintf (stderr, "EXTERNAL \n"); }
		| K_FIELD
			{ fprintf (stderr, "FIELD \n"); }
		| K_FORWARD
			{ fprintf (stderr, "FORWARD \n"); }
		| K_FROM
			{ fprintf (stderr, "FROM \n"); }
		| K_GEQ
			{ fprintf (stderr, "GEQ \n"); }
		| K_GEQA
			{ fprintf (stderr, "GEQA \n"); }
		| K_GEQU
			{ fprintf (stderr, "GEQU \n"); }
		| K_GLOBAL
			{ fprintf (stderr, "GLOBAL \n"); }
		| K_GTR
			{ fprintf (stderr, "GTR \n"); }
		| K_GTRA
			{ fprintf (stderr, "GTRA \n"); }
		| K_GTRU
			{ fprintf (stderr, "GTRU \n"); }
		| K_IF
			{ fprintf (stderr, "IF \n"); }
		| K_INCR
			{ fprintf (stderr, "INCR \n"); }
		| K_INCRA
			{ fprintf (stderr, "INCRA \n"); }
		| K_INCRU
			{ fprintf (stderr, "INCRU \n"); }
		| K_INITIAL
			{ fprintf (stderr, "INITIAL \n"); }
		| K_INRANGE
			{ fprintf (stderr, "INRANGE \n"); }
		| K_IOPAGE
			{ fprintf (stderr, "IOPAGE \n"); }
		| K_KEYWORDMACRO
			{ fprintf (stderr, "KEYWORDMACRO \n"); }
		| K_LABEL
			{ fprintf (stderr, "LABEL \n"); }
		| K_LEAVE
			{ fprintf (stderr, "LEAVE \n"); }
		| K_LEQ
			{ fprintf (stderr, "LEQ \n"); }
		| K_LEQA
			{ fprintf (stderr, "LEQA \n"); }
		| K_LEQU
			{ fprintf (stderr, "LEQU \n"); }
		| K_LIBRARY
			{ fprintf (stderr, "LIBRARY \n"); }
		| K_LINKAGE
			{ fprintf (stderr, "LINKAGE \n"); }
		| K_LITERAL
			{ fprintf (stderr, "LITERAL \n"); }
		| K_LOCAL
			{ fprintf (stderr, "LOCAL \n"); }
		| K_LONG
			{ fprintf (stderr, "LONG \n"); }
		| K_LSS
			{ fprintf (stderr, "LSS \n"); }
		| K_LSSA
			{ fprintf (stderr, "LSSA \n"); }
		| K_LSSU
			{ fprintf (stderr, "LSSU \n"); }
		| K_MACRO
			{ fprintf (stderr, "MACRO \n"); }
		| K_MAP
			{ fprintf (stderr, "MAP \n"); }
		| K_MOD
			{ fprintf (stderr, "MOD \n"); }
		| K_MODULE
			{ fprintf (stderr, "MODULE \n"); }
		| K_NEQ
			{ fprintf (stderr, "NEQ \n"); }
		| K_NEQA
			{ fprintf (stderr, "NEQA \n"); }
		| K_NEQU
			{ fprintf (stderr, "NEQU \n"); }
		| K_NOT
			{ fprintf (stderr, "NOT \n"); }
		| K_NOVALUE
			{ fprintf (stderr, "NOVALUE \n"); }
		| K_OF
			{ fprintf (stderr, "OF \n"); }
		| K_OR
			{ fprintf (stderr, "OR \n"); }
		| K_OTHERWISE
			{ fprintf (stderr, "OTHERWISE \n"); }
		| K_OUTRANGE
			{ fprintf (stderr, "OUTRANGE \n"); }
		| K_OWN
			{ fprintf (stderr, "OWN \n"); }
		| K_PLIT
			{ fprintf (stderr, "PLIT \n"); }
		| K_PRESET
			{ fprintf (stderr, "PRESET \n"); }
		| K_PSECT
			{ fprintf (stderr, "PSECT \n"); }
		| K_RECORD
			{ fprintf (stderr, "RECORD \n"); }
		| K_REF
			{ fprintf (stderr, "REF \n"); }
		| K_REGISTER
			{ fprintf (stderr, "REGISTER \n"); }
		| K_REP
			{ fprintf (stderr, "REP \n"); }
		| K_REQUIRE
			{ fprintf (stderr, "REQUIRE \n"); }
		| K_RETURN
			{ fprintf (stderr, "RETURN \n"); }
		| K_ROUTINE
			{ fprintf (stderr, "ROUTINE \n"); }
		| K_SELECT
			{ fprintf (stderr, "SELECT \n"); }
		| K_SELECTA
			{ fprintf (stderr, "SELECTA \n"); }
		| K_SELECTONE
			{ fprintf (stderr, "SELECTONE \n"); }
		| K_SELECTONEA
			{ fprintf (stderr, "SELECTONEA \n"); }
		| K_SELECTONEU
			{ fprintf (stderr, "SELECTONEU \n"); }
		| K_SELECTU
			{ fprintf (stderr, "SELECTU \n"); }
		| K_SET
			{ fprintf (stderr, "SET \n"); }
		| K_SHOW
			{ fprintf (stderr, "SHOW \n"); }
		| K_SIGNED
			{ fprintf (stderr, "SIGNED \n"); }
		| K_STACKLOCAL
			{ fprintf (stderr, "STACKLOCAL \n"); }
		| K_STRUCTURE
			{ fprintf (stderr, "STRUCTURE \n"); }
		| K_SWITCHES
			{ fprintf (stderr, "SWITCHES \n"); }
		| K_TES
			{ fprintf (stderr, "TES \n"); }
		| K_THEN
			{ fprintf (stderr, "THEN \n"); }
		| K_TO
			{ fprintf (stderr, "TO \n"); }
		| K_UNDECLARE
			{ fprintf (stderr, "UNDECLARE \n"); }
		| K_UNSIGNED
			{ fprintf (stderr, "UNSIGNED \n"); }
		| K_UNTIL
			{ fprintf (stderr, "UNTIL \n"); }
		| K_UPLIT
			{ fprintf (stderr, "UPLIT \n"); }
		| K_VOLATILE
			{ fprintf (stderr, "VOLATILE \n"); }
		| K_WEAK
			{ fprintf (stderr, "WEAK \n"); }
		| K_WHILE
			{ fprintf (stderr, "WHILE \n"); }
		| K_WITH
			{ fprintf (stderr, "WITH \n"); }
		| K_WORD
			{ fprintf (stderr, "WORD \n"); }
		| K_XOR
			{ fprintf (stderr, "XOR \n"); }
		| P_ALLOCATION
			{ fprintf (stderr, "%%ALLOCATION \n"); }
		| P_ASCIC
			{ fprintf (stderr, "%%ASCIC \n"); }
		| P_ASCID
			{ fprintf (stderr, "%%ASCID \n"); }
		| P_ASCII
			{ fprintf (stderr, "%%ASCII \n"); }
		| P_ASCIZ
			{ fprintf (stderr, "%%ASCIZ \n"); }
		| P_ASSIGN
			{ fprintf (stderr, "%%ASSIGN \n"); }
		| P_B
			{ fprintf (stderr, "%%B \n"); }
		| P_BLISS
			{ fprintf (stderr, "%%BLISS \n"); }
		| P_BLISS16
			{ fprintf (stderr, "%%BLISS16 \n"); }
		| P_BLISS32
			{ fprintf (stderr, "%%BLISS32 \n"); }
		| P_BLISS36
			{ fprintf (stderr, "%%BLISS36 \n"); }
		| P_BPADDR
			{ fprintf (stderr, "%%BPADDR \n"); }
		| P_BPUNIT
			{ fprintf (stderr, "%%BPUNIT \n"); }
		| P_BPVAL
			{ fprintf (stderr, "%%BPVAL \n"); }
		| P_C
			{ fprintf (stderr, "%%C \n"); }
		| P_CHAR
			{ fprintf (stderr, "%%CHAR \n"); }
		| P_CHARCOUNT
			{ fprintf (stderr, "%%CHARCOUNT \n"); }
		| P_CHESC
			{ fprintf (stderr, "%%CHESC \n"); }
		| P_COUNT
			{ fprintf (stderr, "%%COUNT \n"); }
		| P_CTCE
			{ fprintf (stderr, "%%CTCE \n"); }
		| P_D
			{ fprintf (stderr, "%%D \n"); }
		| P_DECIMAL
			{ fprintf (stderr, "%%DECIMAL \n"); }
		| P_DECLARED
			{ fprintf (stderr, "%%DECLARED \n"); }
		| P_E
			{ fprintf (stderr, "%%E \n"); }
		| P_ELSE
			{ fprintf (stderr, "%%ELSE \n"); }
		| P_ERROR
			{ fprintf (stderr, "%%ERROR \n"); }
		| P_ERRORMACRO
			{ fprintf (stderr, "%%ERRORMACRO \n"); }
		| P_EXACTSTRING
			{ fprintf (stderr, "%%EXACTSTRING \n"); }
		| P_EXITITERATION
			{ fprintf (stderr, "%%EXITITERATION \n"); }
		| P_EXITMACRO
			{ fprintf (stderr, "%%EXITMACRO \n"); }
		| P_EXPAND
			{ fprintf (stderr, "%%EXPAND \n"); }
		| P_EXPLODE
			{ fprintf (stderr, "%%EXPLODE \n"); }
		| P_FI
			{ fprintf (stderr, "%%FI \n"); }
		| P_FIELDEXPAND
			{ fprintf (stderr, "%%FIELDEXPAND \n"); }
		| P_G
			{ fprintf (stderr, "%%G \n"); }
		| P_H
			{ fprintf (stderr, "%%H \n"); }
		| P_IDENTICAL
			{ fprintf (stderr, "%%IDENTICAL \n"); }
		| P_IF
			{ fprintf (stderr, "%%IF \n"); }
		| P_INFORM
			{ fprintf (stderr, "%%INFORM \n"); }
		| P_ISSTRING
			{ fprintf (stderr, "%%ISSTRING \n"); }
		| P_LENGTH
			{ fprintf (stderr, "%%LENGTH \n"); }
		| P_LTCE
			{ fprintf (stderr, "%%LTCE \n"); }
		| P_MESSAGE
			{ fprintf (stderr, "%%MESSAGE \n"); }
		| P_NAME
			{ fprintf (stderr, "%%NAME \n"); }
		| P_NBITS
			{ fprintf (stderr, "%%NBITS \n"); }
		| P_NBITSU
			{ fprintf (stderr, "%%NBITSU \n"); }
		| P_NULL
			{ fprintf (stderr, "%%NULL \n"); }
		| P_NUMBER
			{ fprintf (stderr, "%%NUMBER \n"); }
		| P_O
			{ fprintf (stderr, "%%O \n"); }
		| P_P
			{ fprintf (stderr, "%%P \n"); }
		| P_PRINT
			{ fprintf (stderr, "%%PRINT \n"); }
		| P_QUOTE
			{ fprintf (stderr, "%%QUOTE \n"); }
		| P_QUOTENAME
			{ fprintf (stderr, "%%QUOTENAME \n"); }
		| P_RAD50_10
			{ fprintf (stderr, "%%RAD50_10 \n"); }
		| P_RAD50_11
			{ fprintf (stderr, "%%RAD50_11 \n"); }
		| P_REF
			{ fprintf (stderr, "%%REF \n"); }
		| P_REMAINING
			{ fprintf (stderr, "%%REMAINING \n"); }
		| P_REMOVE
			{ fprintf (stderr, "%%REMOVE \n"); }
		| P_REQUIRE
			{ fprintf (stderr, "%%REQUIRE \n"); }
		| P_SBTTL
			{ fprintf (stderr, "%%SBTTL \n"); }
		| P_SIXBIT
			{ fprintf (stderr, "%%SIXBIT \n"); }
		| P_SIZE
			{ fprintf (stderr, "%%SIZE \n"); }
		| P_STRING
			{ fprintf (stderr, "%%STRING \n"); }
		| P_SWITCHES
			{ fprintf (stderr, "%%SWITCHES \n"); }
		| P_THE
			{ fprintf (stderr, "%%THE \n"); }
		| P_TITLE
			{ fprintf (stderr, "%%TITLE \n"); }
		| P_UNQUOTE
			{ fprintf (stderr, "%%UNQUOTE \n"); }
		| P_UPVAL
			{ fprintf (stderr, "%%UPVAL \n"); }
		| P_VARIANT
			{ fprintf (stderr, "%%VARIANT \n"); }
		| P_WARN
			{ fprintf (stderr, "%%WARN \n"); }
		| P_X
			{ fprintf (stderr, "%%X \n"); }
		;

%%


#include "lex.yy.c"

/*
support
functions
here
*/



yyerror ()
{
fprintf (stderr, "Nu b;lev det fel\n");
}
